<?php

declare(strict_types=1);

namespace Drupal\externalauth_force\EventSubscriber;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Session\AnonymousUserSession;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\externalauth\AuthmapInterface;
use Drupal\user\UserInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Automatically logs out authenticated users on public domain.
 */
final readonly class RequestSubscriber implements EventSubscriberInterface {

  /**
   * RequestSubscriber constructor.
   */
  public function __construct(
    protected AccountProxyInterface $currentUser,
    protected SessionManagerInterface $sessionManager,
    protected AuthmapInterface $authmap,
  ) {}

  /**
   * Acts on KernelEvents::REQUEST.
   */
  public function onRequest(): void {
    $this->checkSession($this->currentUser);
  }

  /**
   * Acts on "hook_user_login".
   */
  public function onLogin(UserInterface $account): void {
    // Invalidate new user sessions not having gone through External
    // Authentication.
    $this->checkSession($account);
  }

  /**
   * Checks the current user session, destroying it, if needed.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   A user account.
   */
  protected function checkSession(AccountInterface $account): void {
    if ($account->isAuthenticated() && !$account->hasPermission('skip external authentication') && !$this->authmap->getAll($account->id())) {
      if ($this->sessionManager->isStarted()) {
        $this->sessionManager->destroy();
      }
      $this->sessionManager->delete($account->id());
      $this->currentUser->setAccount(new AnonymousUserSession());
      throw new AccessDeniedHttpException('Authenticated access is not allowed');
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::REQUEST][] = ['onRequest'];
    return $events;
  }

}
